@@ -1,11 +1,12 @@
 import os.path
 
+import gribapi  # type: ignore
 import numpy as np
 import pytest
 
 xr = pytest.importorskip("xarray")  # noqa
 
-from cfgrib import xarray_store
+from cfgrib import dataset, xarray_store
 
 SAMPLE_DATA_FOLDER = os.path.join(os.path.dirname(__file__), "sample-data")
 TEST_DATA = os.path.join(SAMPLE_DATA_FOLDER, "era5-levels-members.grib")
@@ -13,8 +14,10 @@ TEST_CORRUPTED = os.path.join(SAMPLE_DATA_FOLDER, "era5-levels-corrupted.grib")
 TEST_DATASETS = os.path.join(SAMPLE_DATA_FOLDER, "t_on_different_level_types.grib")
 TEST_IGNORE = os.path.join(SAMPLE_DATA_FOLDER, "uv_on_different_levels.grib")
 TEST_DATA_NCEP_MONTHLY = os.path.join(SAMPLE_DATA_FOLDER, "ncep-seasonal-monthly.grib")
+TEST_DATA_MULTIPLE_FIELDS = os.path.join(SAMPLE_DATA_FOLDER, "regular_gg_ml_g2.grib")
 TEST_DATA_DIFFERENT_STEP_TYPES = os.path.join(SAMPLE_DATA_FOLDER, "cfrzr_and_cprat.grib")
 TEST_DATA_DIFFERENT_STEP_TYPES_ZEROS = os.path.join(SAMPLE_DATA_FOLDER, "cfrzr_and_cprat_0s.grib")
+TEST_DATA_ALTERNATE_ROWS_MERCATOR = os.path.join(SAMPLE_DATA_FOLDER, "ds.waveh.5.grib")
 
 
 def test_open_dataset() -> None:
@@ -38,9 +41,11 @@ def test_open_dataset() -> None:
     res = xarray_store.open_dataset(TEST_IGNORE, backend_kwargs={"errors": "ignore"})
     assert "isobaricInhPa" in res.dims
 
-    with pytest.raises(ValueError):
+    with pytest.raises(dataset.DatasetBuildError):
         xarray_store.open_dataset(TEST_IGNORE, backend_kwargs={"errors": "raise"})
 
+    xarray_store.open_dataset(TEST_DATA, backend_kwargs={"errors": "raise"})
+
 
 def test_open_dataset_corrupted() -> None:
     res = xarray_store.open_dataset(TEST_CORRUPTED)
@@ -48,8 +53,8 @@ def test_open_dataset_corrupted() -> None:
     assert res.attrs["GRIB_edition"] == 1
     assert len(res.data_vars) == 1
 
-    with pytest.raises(Exception):
-        xarray_store.open_dataset(TEST_CORRUPTED, backend_kwargs={"grib_errors": "raise"})
+    with pytest.raises(gribapi.GribInternalError):
+        xarray_store.open_dataset(TEST_CORRUPTED, backend_kwargs={"errors": "raise"})
 
 
 def test_open_dataset_encode_cf_time() -> None:
@@ -122,6 +127,14 @@ def test_open_datasets() -> None:
     assert res[0].attrs["GRIB_centre"] == "ecmf"
 
 
+def test_cached_geo_coords() -> None:
+    ds1 = xarray_store.open_dataset(TEST_DATA_MULTIPLE_FIELDS)
+    ds2 = xarray_store.open_dataset(
+        TEST_DATA_MULTIPLE_FIELDS, backend_kwargs=dict(cache_geo_coords=False)
+    )
+    assert ds2.identical(ds1)
+
+
 def test_open_datasets_differet_step_types() -> None:
     res = xarray_store.open_datasets(TEST_DATA_DIFFERENT_STEP_TYPES)
 
@@ -142,3 +155,11 @@ def test_open_datasets_differet_step_types_zeros() -> None:
     assert res[0].cfrzr.attrs["GRIB_stepType"] == "instant"
     assert res[1].cprat.attrs["GRIB_stepType"] == "avg"
     assert res[1].cfrzr.attrs["GRIB_stepType"] == "avg"
+
+
+def test_alternating_scanning_mercator() -> None:
+    ds = xarray_store.open_dataset(TEST_DATA_ALTERNATE_ROWS_MERCATOR)
+    values = ds.variables["shww"].data
+    assert np.isnan(values[5])
+    assert values[760500] == 1.5
+    values_all = ds.variables["shww"].data[:]
