@@ -7,7 +7,7 @@ import pytest
 
 xr = pytest.importorskip("xarray")  # noqa
 
-from cfgrib import xarray_store, xarray_to_grib
+from cfgrib import dataset, xarray_store, xarray_to_grib
 
 SAMPLE_DATA_FOLDER = os.path.join(os.path.dirname(__file__), "sample-data")
 
@@ -49,7 +49,7 @@ def test_open_dataset(grib_name: str) -> None:
 def test_open_dataset_fail(grib_name: str) -> None:
     grib_path = os.path.join(SAMPLE_DATA_FOLDER, grib_name + ".grib")
 
-    with pytest.raises(ValueError):
+    with pytest.raises(dataset.DatasetBuildError):
         xarray_store.open_dataset(grib_path, cache=False, backend_kwargs={"errors": "raise"})
 
 
